"""
coinfee library for http://coinfee.net/

Released into the public domain.
"""

from collections import namedtuple
from warnings import warn
import json
import urllib2

import yaml


COINFEE_URL = 'https://coinfee.net/payment'

TIMEOUT = 60


def payment(address,
            satoshis,
            unique,
            fee_address=None,
            fee=None,
            coinfee_url=COINFEE_URL):
    """
    Processes a payment with coinfee.

    Returns:
    coinfee_payment.address which is the address to pay.
    coinfee_payment.satoshis which is the total amount that needs to be paid.
    coinfee_payment.status True/False, if payment has been finished.
    coinfee_payment.just_paid True/False, if sent in the current request.
    """
    pre_data = {'address': address,
                'satoshis': satoshis,
                'unique': unique}
    if fee_address is not None and fee is not None:
        pre_data['fee_address'] = fee_address
        pre_data['fee'] = fee
    post_data = json.dumps(pre_data)
    try:
        http_return = urllib2.urlopen(coinfee_url,
                                      data=post_data,
                                      timeout=TIMEOUT)
    except urllib2.HTTPError as http_error:
        if http_error.code == 400:
            # Throw exception with output from coinfee.
            raise Exception(http_error.read())
        else:
            raise
    if http_return.getcode() == 200:
        data = yaml.safe_load(http_return.read())
        if 'deprecated' in data:
            if data['deprecated'] is not False:
                warn(str(data['deprecated']), DeprecationWarning)
        coinfee_payment = namedtuple('coinfee_payment',
                                     ['address',
                                      'satoshis'
                                      'status'])
        coinfee_payment.address = data['address']
        coinfee_payment.satoshis = data['satoshis']
        coinfee_payment.status = data['status']
        coinfee_payment.just_paid = data['just_paid']
        return coinfee_payment
    else:
        raise Exception('Fatal issue with coinfee.')
